# Copyright 2017 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging

from almanach.collector.filters import base_filter

LOG = logging.getLogger(__name__)


class ErroredInstanceFilter(base_filter.BaseFilter):

    def ignore_notification(self, notification):
        if notification.payload.get('state') == 'error':
            LOG.info('Instance deletion event ignored because instance %s was badly created',
                     notification.payload.get('instance_id'))
            return True
        return False
